#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
    Module name: timebomb
    Coded by: Michele '0blio' Cisternino
    Version: 0.4

    Description:

        This module will do various guesses of possible filenames by using timestamps.
        All the permutations will be calculated, even if you use custom words.
        All the possibile filenames will be calculated in a range of N minutes before
        the start of the script.
"""

import os
from itertools import permutations
import datetime, time
from Helpers.utilities import *

output = []

# Taking the name and the ext of the file you uploaded
fname, fext = os.path.splitext(filename)

# Taking system timestamp
verbose_message (notification ("Reading local datetime..", "added", False), verbose)
now = datetime.datetime.now().replace(microsecond=0)
verbose_message (notification ("Local datetime: %s" % (now), "added", False), verbose)

# Taking the remote system timestamp
verbose_message (notification ("Reading target datetime..", "added", False), verbose)
now_remote = get_remote_timestamp (base_url)
if now_remote == False:
    verbose_message (notification ("Unable to read target datetime. Only the local datetime will be used.", "error", False), verbose)
else:
    verbose_message (notification ("Target datetime: %s" % (now_remote), "added", False), verbose)

timestamps = [now]

# N mins to check before the timestamp
mins = 1

# If difference in minutes between local and remote timestamp is greater than mins add the remote timestamp to computation
if now_remote == True and minutes_between_timestamps(now, now_remote) >= mins:
    timestamps.insert(0, now_remote)

# Declaring the possibile filename formats
verbose_message (notification ("Hashing '%s' in various ways.." % (filename), "added", False), verbose)
filename_formats = [fname, fname + fext, string_to_md5(fname), string_to_sha1(fname), string_to_sha256(fname)]

for timestamp in timestamps:

    #verbose_message (notification ("Iterating timestamps up to %s minutes ago.." % (str(mins)), "added", False), verbose)

    # Take timestamp of N mins before now
    timestamp_minus_n_mins = timestamp - datetime.timedelta(minutes = mins)

    verbose_message (notification ("Iterating timestamps from '%s' to '%s' (%s mins)" % (timestamp, timestamp_minus_n_mins, str(mins)), "added", False), verbose)
    verbose_message (notification ("Calculating all possible combinations of filenames, timestamps and custom words provided as input (if any)", "added", False), verbose)

    # Timestamp taken at the start of the script
    tmp_timestamp = timestamp

    # Iterate N minutes before the timestamp taken at the start of the script
    while tmp_timestamp != timestamp_minus_n_mins:
        permutation_items = []

        # If the user defined any custom words add them to the permutation items
        if len(custom_words) > 0:
            permutation_items += custom_words

        # Adding the uploaded filename to permutation items
        for i, filename_format in enumerate(filename_formats):
            # If it is the first iteration the filename has to be inserted in the first place
            if i == 0:
                permutation_items.insert(0, filename_format)

            # Else the filename has to be replaced
            else:
                permutation_items[0] = filename_format

            # Iterate over different timestamp_format
            timestamp_formats = timestamp_all_formats(tmp_timestamp)
            for timestamp_format in timestamp_formats:
                output.append(timestamp_format + fext)

                # Adding the formatted timestamp to the permutation item list
                permutation_items.append(timestamp_format)

                # zipped_combinations (permutation_items, ["_", "-", "", "."], fext)
                output += zipped_combinations (permutation_items, ["_", "-", "", "."], fext, True)

                # Deleting the actual formatted timestamp
                del permutation_items[-1]

        # Subtract one second to the current timestamp
        tmp_timestamp -= datetime.timedelta(seconds=1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
    Module name: shame
    Coded by: Michele '0blio' Cisternino
    Version: 0.4

    Description:

        This modules generate different hashed version of the filenames.
        If the user don't specify a specific path to search the file in, it append the generated values to a list of common upload directories.
"""

output = []
common_dirs = []

verbose_message (notification ("Reading a list of common upload directories..", "added", False), verbose)
common_dirs = open("txt/common-directories.txt", "r").read().splitlines()
verbose_message (notification ("%s directories read!" % (str(len(common_dirs))), "added", False), verbose)

# Taking the name and the ext of the file the user uploaded
fname, fext = os.path.splitext(filename)

# Creating a list with the clear-text filename and various filename hashes
verbose_message (notification ("Hashing '%s' in various ways.." % (filename), "added", False), verbose)
hashed_filenames = [o + fext for o in [fname, string_to_md5(fname), string_to_md5(fname + fext), string_to_sha1(fname), string_to_sha256(fname)]]

# Output hashed_filenames
output += hashed_filenames

# If the user specified some custom words to computate
if len(custom_words) > 0:
    verbose_message (notification ("Calculating all the possible combinations of filename using the custom words provided as input", "added", False), verbose)

    # Generate all the possible permutations with the separators
    permutation_items = [fname] + custom_words

    output += zipped_combinations (permutation_items, ["_", "-", "", "."], fext)

# Output common directories + hashed_filenames
if common_dirs != []:
    verbose_message (notification ("Appending generated values to the common upload directories wordlist..", "added", False), verbose)

output += [dir + f for dir in common_dirs for f in output]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys

class bcolors:
    PINK = '\033[95m'
    BLUE = '\033[94m'
    GREEN = '\033[92m'
    YELLOW = '\033[93m'
    RED = '\033[91m'
    BOLD = '\033[1m'
    ENDC = '\033[0m'

def print_logo ():
    print """
    \033[97m███████╗██╗██╗     ███████╗\033[91m ██████╗ ██████╗ ███████╗\033[0m
    \033[97m██╔════╝██║██║     ██╔════╝\033[91m██╔════╝ ██╔══██╗██╔════╝\033[0m
    \033[97m█████╗  ██║██║     █████╗  \033[91m██║  ███╗██████╔╝███████╗\033[0m
    \033[97m██╔══╝  ██║██║     ██╔══╝  \033[91m██║   ██║██╔═══╝ ╚════██║\033[0m
    \033[97m██║     ██║███████╗███████╗\033[91m╚██████╔╝██║     ███████║\033[0m
    \033[97m╚═╝     ╚═╝╚══════╝╚══════╝\033[91m ╚═════╝ ╚═╝     ╚══════╝\033[0m \033[95m{v0.4}
    \033[93m                   Coded by 0blio                   \033[0m
                  \033[94m\033[4mhttps://github.com/0blio\033[0m

\033[4m\033[1m\033[91mLegal disclaimer\033[0m
Usage of fileGPS for attacking targets without prior mutual consent is illegal.
It is the end user's responsibility to obey all applicable local, state and federal laws.
Developers are not responsible for any misuse or damage caused by this software.
    """

def notification (text, type, verbose = True):
    symbol = bcolors.BOLD

    if type == "added":
        symbol += bcolors.PINK + "+"
    elif type == "success":
        symbol += bcolors.GREEN + "✓"
    elif type == "removed":
        symbol += bcolors.RED + "-"
    elif type == "notify":
        symbol += bcolors.BLUE + "★"
    elif type == "error":
        symbol += bcolors.RED + "✗"
    elif type == "found":
        symbol += bcolors.RED + "♥"
    elif type == "info":
        symbol += bcolors.YELLOW + "INFO"

    symbol += bcolors.ENDC

    if verbose == True:
        print "[" + symbol + "] " + text
    return "[" + symbol + "] " + text

def verbose_message (text, verbose = False):
    if verbose == True:
        print text

def print_overriding (values):
    for value in values:
        print value

        # Back to the previous line
        sys.stdout.write("\033[F")

        # Clear line
        sys.stdout.write("\033[K")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import hashlib
import datetime, time
from graphic import *
from itertools import permutations, chain, izip_longest, product

def string_to_md5 (string):
    m = hashlib.md5()
    m.update(string)
    return m.hexdigest()

def string_to_sha1 (string):
    m = hashlib.sha1()
    m.update(string)
    return m.hexdigest()

def string_to_sha256 (string):
    m = hashlib.sha256()
    m.update (string)
    return m.hexdigest()

def percentage(part, whole):
    return format(100 * float(part)/float(whole), '.2f')

def timestamp_all_formats (timestamp):
    timestamps = []
    format_strings = ["%Y%m%d%H%M%S", "%d%m%Y%H%M%S", "PHP_TIME"]

    for format_string in format_strings:
        if format_string == "PHP_TIME":
            formatted_string = str(int(time.mktime(timestamp.timetuple())))
        else:
            formatted_string = time.strftime(format_string, timestamp.timetuple())

        timestamps.append(formatted_string)
        timestamps.append(string_to_md5(formatted_string))

    return timestamps

# This function create combinations merging two lists.
def zipped_combinations (a, b, append = "", md5 = False):
    output = []

    for perm in permutations (a):
        for prod in product (b, repeat=len(a) - 1):
            tpls = list(chain.from_iterable(izip_longest(perm, prod, fillvalue='')))
            output.append (''.join(tpls) + append)

            if md5 == True:
                output.append (string_to_md5(''.join(tpls)) + append)

    return output

def minutes_between_timestamps (t1, t2):
    difference = t1 - t2 if t1 > t2 else t2 - t1
    return int(round(difference.total_seconds() / 60))

def question_yn (question):
    while 1:
        try:
            proceed = ""
            proceed = raw_input (question + "? [y/n]: ").lower().replace(" ", "")
            if proceed in ['y', 'n', ""]:
                return proceed
        except:
            print
            notification ("Stopped\n", "removed")
            exit()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import sys
import requests
import os
from multiprocessing import Pool, Queue
from Helpers.graphic import *
from Helpers.connections import *
from Helpers.utilities import *
from functools import partial
import Helpers
import urllib3

# Disable SSL Warnings
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

# Don't write .pyc files
sys.dont_write_bytecode = True

# Formatter class for argparse
formatter = lambda prog: argparse.HelpFormatter(prog, max_help_position=35, width=95)

# Initializing the parser
parser = argparse.ArgumentParser (description=print_logo(), usage="python %s -u <url[/path]> --file <filename> [options]" % sys.argv[0], add_help=False, formatter_class=formatter)

# Removing the default arguments group (to customize the --help parameter)
parser._action_groups.pop()

# Creating three groups of arguments (required arguments, optional arguments and modules)
required = parser.add_argument_group ("\033[1mRequired arguments\033[0m")
optional = parser.add_argument_group ("\033[1mOptional arguments\033[0m")

# These are not valid arguments, it is just to print usefull data in a formatted way
modules = parser.add_argument_group ("\033[1mModules\033[0m")
examples = parser.add_argument_group ("\033[1mExamples\033[0m")

# Setting up arguments in the "required" group
required.add_argument ("-u", "--url", metavar=("URL"), help="Target URL (e.g \"http://target.com/uploads\")", required=True)
required.add_argument ("-f", "--file", metavar=("FILENAME"), help="Filename of the file uploaded on the target server", required=True)

# Setting up arguments in the "optional" group
optional.add_argument ("--modules", metavar=("MODULE1,[MODULE2..]"), help="Comma separated list of the modules to be imported (Use '*' to import all modules). By default, all modules will be used.", default="*")
optional.add_argument ("--proxy", metavar=("PROXY"), help="Use a proxy to connect to the target URL")
optional.add_argument ("--custom", metavar=("WORDS"), help="Comma separated list of the custom words to be used to find the file")
optional.add_argument ("--cookie", metavar=("COOKIE"), help="HTTP Cookie header value")
optional.add_argument ("--threads", metavar=("THREADS"), help="Max number of concurrent HTTP(s) requests (default: 1", type=int, default=1)
optional.add_argument ("--user-agent", metavar=('AGENT'), help="HTTP User-Agent header value")
optional.add_argument ("--match", metavar=('STRING'), help="Text pattern to match in the response (e.g. webshell)")
optional.add_argument ("--random-agent", action="store_true", help="Use randomly selected HTTP User-Agent header value")
optional.add_argument ("-o", "--output", metavar=("FILENAME"), help="Save all generated filenames in a file")
optional.add_argument ("-v", "--verbose", action="store_true", help="Verbose mode")
optional.add_argument ("-h", "--help", help="Show help message and exit")

# Setting up the list of available modules
modules.add_argument ("shame", action="store_true", help="Generates a list of hashed filenames. If the user doesn't specify a specific path to search the file in, it appends the generated values to a list of common upload directories.")
modules.add_argument ("timebomb", action="store_true", help="Generates various combinations of timestamps and filenames iterating up to 1 minute before the start of the script.")
modules.add_argument ("monkey", action="store_true", help="Generates various combinations of filenames, sequential numbers and custom words.")

# If there aren't arguments passed to the program  or user requested an help print the help message and exit
if len(sys.argv[1:]) == 0 or len(sys.argv[1:]) == 1 and sys.argv[1] in ["-h", "--help"]:
	parser.print_help()
	print
	exit()

# Parsing the arguments
args = parser.parse_args()

# Assigning all used arguments to respective data structures
url = make_host_valid(args.url)
base_url = get_base_hostname (url)
filename = args.file
modules = args.modules.replace(" ", "")
custom_words = args.custom.replace(" ", "").split(",") if args.custom else []
session_cookie = args.cookie if args.cookie else ""
proxy = args.proxy if args.proxy else ""
file_output = args.output if args.output else ""
threads = args.threads
match = args.match if args.match else ""
user_agent = args.user_agent if args.user_agent else ""
random_agent = True if args.random_agent else False
verbose = True if args.verbose else False

if random_agent:
	notification ("Setting up random user agent..", "info")
	user_agent = random_user_agent()
	notification ("Selected user agent: %s" % (user_agent), "success")

elif args.user_agent == "" or not args.user_agent or not random_agent:
	verbose_message (notification("No user agent specified.\n", "added", False), verbose)
	try:
		notification ("Setting up default user agent..", "info")
		user_agent = "fileGPS_0blio"
		notification ("Selected user agent: %s" % (user_agent), "success")
	except:
		notification ("Error reading the user agents file.", "error")

print

# Checking if the target is up and running
notification ("Testing connection with the target.", "info")
verbose_message (notification("Sending an HTTP HEAD request to %s .." % base_url, "added", False), verbose)

if check_target_connection (url, user_agent):
    notification ("The target appears to be up and running!", "success")
else:
    notification ("The target seems to be down. Quitting..\n", "error")
    exit()

print

# If the user want to import all modules (or he didn't specified any modules)
if modules == "*":
    modules = []

    # If there are no modules
    if len(os.listdir("Modules")) == 0:
        notification ("No modules found. Quitting..", "error")
        exit()

    # Add all .py contained in Modules/ to the list of modules to use
    for m in os.listdir ("Modules"):
        if m.endswith (".py"):
            modules.append(m)

    # Sort modules by relevance
    tmp = []
    for r in ["shame.py", "timebomb.py", "monkey.py"]:
        if r in modules:
            tmp.append(r)
    modules = tmp + [x for x in modules if x not in ['shame.py', 'timebomb.py', 'monkey.py']]
    del tmp

# Else if the user specified the modules to import
else:
    argument_values = modules.split (",")

	# If there are some duplicates specified by the user filegps will not re-execute an already executed module
    if len(argument_values) != len(set(argument_values)):
		verbose_message (notification("You have selected duplicates modules. Removing them automatically..\n", "added", False), verbose)
		argument_values = list(dict.fromkeys(argument_values))

    argument_values = [s + ".py" for s in argument_values]
    modules = []

    # If the module exists append it to the list of valid modules
    for argument in argument_values:
        if os.path.isfile ("Modules/" + argument):
            modules.append(argument)
        else:
			notification ("Module '%s' not found\n" % argument.replace(".py", ""), "error")

    del argument_values

# Checking if there some modules
if len(modules) == 0:
	notification ("No modules found. Quitting..\n", "error")
	exit()
else:
	notification ("Running all %s valid modules." % str(len(modules)), "info")

# Executing valid modules
filenames = []
for module in modules:
    notification ("Running module '%s' .." % module.replace(".py", ""), "notify")
    execfile ("Modules/" + module)

    # Checking if the module generated an output (allocated list of filenames to search for)
    if 'output' in locals():
        notification ("'%s' generated \033[1m%s\033[0m possible filenames.\n" % (module.replace(".py", ""), str(len(output))), "success")
        filenames += output
        del output

    else:
        notification ("Invalid module: '%s'\n" % module.replace(".py", ""), "error")

# If the modules generated some filenames
if len(filenames) > 0:
	# If the user want to output the filenames in a file
	if file_output != "":
		notification ("Writing the generated filenames to '%s'" % file_output, "info")

		try:
			# If the file already exists raise an exception
			if os.path.isfile(file_output) and question_yn ("[\033[1m\033[94m?\033[0m] %s already exists! Do you want to override it" % (file_output)) == 'n':
				raise Exception ('test')

			with open (file_output, "w") as fp:
				for filename in filenames:
					fp.write("%s\n" % filename)

			verbose_message (notification("Number of lines: %s" % str(len(filenames)), "added", False), verbose)
			notification ("File written successfully!\n", "success")
		except:
			notification ("Error writing the file!\n", "error")
        else:
                verbose_message(notification("No output file specified. Proceeding without saving filenames..\n", "added", False), verbose)

	# Asking to the user if he wants to proceed with the test
	if question_yn ("[\033[1m\033[94m?\033[0m] Do you want to test %s filenames on %s" % (str(len(filenames)), url)) == 'n':
		notification ("Stopped\n", "removed")
		exit()

	print

	notification ("Testing all %s generated filenames with %s threads.." % (str(len(filenames)), threads), "info")

	# Allocating shared memory between processes
	queue = Queue()

	# Allocating a pool of N threads
	p = Pool (threads, worker_init, [queue])

	# Creating a partial function with the fixed arguments
	r = partial (requester, url, session_cookie, user_agent, proxy, match)

	# Mapping asynchronously the function on the spawned processes
	workers = p.map_async (r, filenames)

	i = 0
	len_filenames = len(filenames)

	# Accessing the shared memory until something happen
	while 1:
		try:
			if i == len_filenames:
				notification ("File not found. Have you an idea on how to find it? Ask for a feature on \033[94m\033[4mhttps://github.com/0blio/filegps\033[0m.\n", "error")
				break

			# Getting a result from the shared memory
			result = queue.get()

			print "[%s%s%s] %s" % (bcolors.PINK, percentage(i, len_filenames) + "%", bcolors.ENDC, result[0])

			sys.stdout.write("\033[F")
			sys.stdout.write("\033[K")

			if (result[1] == 200 and match == "") or (result[1] == 200 and match != "" and match in result[2]):
				notification ("FileGPS found your file! It is here: \033[4m%s\033[0m.\n" % result[0], "found")
				break
			# Else if fileGPS got a 200 but the response doesn't contain the word to match
			elif result[1] == 200 and match != "" and match not in result[2]:
				notification ("FileGPS found a file (\033[4m%s\033[0m) but the response doesn't contain the word to match." % result[0], "found")


			i += 1
		except KeyboardInterrupt:
			notification ("Stopped", "removed")
			break

	p.terminate()
	p.join()
	exit()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import requests
from Cookie import SimpleCookie
from random import choice
from urlparse import urlparse
from datetime import datetime
from Helpers.graphic import *
from Helpers.utilities import percentage

def make_host_valid (host):
    host = host.strip()

    if not host.startswith("https://") and not host.startswith("http://"):
        host = "http://" + host

    if not host.endswith("/"):
        host = host + "/"

    return host

def get_base_hostname (url):
    url = urlparse (url)
    url = url.scheme + "://" + url.netloc

    return url

def check_target_connection (target, user_agent):
    target = get_base_hostname (target)

    try:
        requests.head(target, timeout=5, verify=False, headers={"User-Agent": user_agent}).status_code
        return True
    except:
        return False

# This function get remote timestamp of the target http server (it store the data as datetime.datetime)
# I can probably find a better way to do this, but I don't want to rewrite the function :((
def get_remote_timestamp (target):
    months = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
    target = get_base_hostname(target)
    dt = []

    try:
        raw_datetime = requests.head(target).headers["Date"].split(" ")
        dt.append(raw_datetime[3]) # Year
        dt.append([n + 1 for n, month in enumerate(months) if month == raw_datetime[2]][0]) # Month
        dt.append(raw_datetime[1]) # Day of the month
        dt.append(raw_datetime[4].split(":")[0]) # Hour
        dt.append(raw_datetime[4].split(":")[1]) # Minutes
        dt.append(raw_datetime[4].split(":")[2]) # Seconds

        # Converting the datetime list to a Python's datetime compatible tuple
        # Microseconds are not needed but required by datetime
        dt.append(0)
        dt = tuple([int(data) for data in dt])
        dt = datetime(*dt)

        return dt
    except:
        return False

# Salvare file di dirsearch con gli user agent ed estrarli da lì, non ha senso tenerli in memoria
def random_user_agent ():
    user_agents = open("txt/user-agents.txt", "r").read().splitlines()
    return choice(user_agents)

def requester (target, cookie, user_agent, proxy, match, filename):
    try:
        url = target + filename

        proxy_dict = {}
        cookie_dict = {}

        if proxy != "":
            proxy_dict = {"http": proxy, "https": proxy}

        # If the user specified a cookie convert it to a requests acceptable dict
        if cookie != "":
            cookie_dict = SimpleCookie()
            cookie_dict.load(cookie)
            cookie = {}

            for key, morse1 in cookie_dict.items():
                cookie[key] = morse1.value

        # If the user want to match a specific string in the response
        if match != "":
            req = requests.get(url, cookies=cookie, headers={"User-Agent": user_agent}, proxies=proxy_dict, verify=False)
            requester.q.put ([url, req.status_code, req.content])
        else:
            req = requests.head(url, cookies=cookie, headers={"User-Agent": user_agent}, proxies=proxy_dict, verify=False)
            requester.q.put ([url, req.status_code])
    except KeyboardInterrupt:
        sys.stdout = open(os.devnull, 'w')
        #sys.stdout.flush()

# Wrapper for the shared memory multiprocessing queue
def worker_init (q):
    requester.q = q



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
    Module name: monkey
    Coded by: Michele '0blio' Cisternino
    Version: 0.1

    Description:

        This module generate a list of combinations by using sequential numbers.
"""

import itertools

output = []
len_number = 3

incremental = True

# Taking the name and the ext of the file the user uploaded
fname, fext = os.path.splitext(filename)

# Creating a list with the clear-text filename and various filename hashes
verbose_message (notification ("Hashing '%s' in various ways.." % (filename), "added", False), verbose)
filename_formats = [fname, string_to_md5(fname), string_to_sha1(fname)]

# Default: generate random numbers by using an incremental value (from 1 to 3)
# TODO: User input to set len_number

# Generate all the possible numbers with incremental cardinality (from 1 to len_number)
numbers = []

if incremental == True:
    counter = 0
    for i in range(1, len_number + 1, 1):
        verbose_message (notification ("Generating all the possible numbers with len %s" % (str(i)), "added", False), verbose)
        for j in itertools.product(range(10), repeat=i):
            numbers.append(''.join(map(str, j)))
            counter += 1

        verbose_message (notification ("Generated numbers: %s." % (str(counter)), "added", False), verbose)
        counter = 0

elif incremental == False:
    verbose_message (notification ("Generating all the possible numbers with len %s" % (str(len_number)), "added", False), verbose)
    for i in itertools.product(range(10), repeat=len_number):
        numbers.append(''.join(map(str, i)))

# Calculating all the possible permutations
verbose_message (notification ("Calculating all possible combinations of filenames, random numbers and custom words provided as input (if any)", "added", False), verbose)
for number in numbers:
    for f in filename_formats:
        output += zipped_combinations ([f, number], ["_", "-", "", "."], fext)
